"""  This file integrates ASISEyes into the GPS environment.
     It adds a button in the the tool bar and a menu item in the Tools
     menu.
"""

##############################################################################
# No user customisations below this line
##############################################################################

# ============================================================================
# Imports
# ----------------------------------------------------------------------------
import GPS
import gtk
import os
import glob
import shutil

# ============================================================================
# Globals
# ----------------------------------------------------------------------------
ASISEyes_toolbar_button = None
ASISEyes_menu_item      = None
ada_spec_extension      = None
ada_body_extension      = None
ada_separate_extension  = None
current_ada_file_name   = None

# ============================================================================
# Methods
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# Invoked to enable ASISEyes tool bar button and menu entry

def enable_ASISEyes ():
   global ASISEyes_toolbar_button
   global ASISEyes_menu_item
   if ASISEyes_toolbar_button == None:
      return
   if ASISEyes_menu_item == None:
      return
   ASISEyes_toolbar_button.set_sensitive (True)
   ASISEyes_menu_item.set_sensitive (True)
   gtk.Tooltips().set_tip(ASISEyes_toolbar_button, "Run ASIS Eyes")

# ----------------------------------------------------------------------------
# Invoked to disable ASISEyes tool bar button and menu entry

def disable_ASISEyes ():
   global ASISEyes_toolbar_button
   global ASISEyes_menu_item

   # Be aware we may be there before all initialization was done

   if ASISEyes_toolbar_button == None:
      return
   if ASISEyes_menu_item == None:
      return

   ASISEyes_toolbar_button.set_sensitive (False)
   ASISEyes_menu_item.set_sensitive (False)
   gtk.Tooltips().set_tip (
      ASISEyes_toolbar_button,
      "ASIS Eyes disabled : requires an Ada source file context")

# ----------------------------------------------------------------------------
# To know if we should enable/disable ASISEyes command on a file context
# Used in on_GPS_context_changed

def is_an_ada_file (file_name):
   global ada_spec_extension
   global ada_body_extension
   global ada_separate_extension
   
   # Be aware we may be there before all initialization was done

   if ada_spec_extension == None:
      return False
   if ada_body_extension == None:
      return False
   if ada_separate_extension == None:
      return False
      
   current_file_extension = os.path.splitext (file_name)[1]
   return (
      (current_file_extension == ada_spec_extension) or
      (current_file_extension == ada_body_extension) or
      (current_file_extension == ada_separate_extension))

# ============================================================================
# UI event handlers
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# Handle the ASISEyes command

def on_click_run_asiseyes (X):

   global current_ada_file_name

   # Be aware we may be there before all initialization was done

   if current_ada_file_name == None:
      return

   tree_file_extension_filter = "*.adt"
   project = GPS.Project.root()
   project_name = project.name()
   objects_directory = project.get_attribute_as_string ("Object_Dir")

   GPS.MDI.save_all ()
   # Save all opened file to ensure compilation will be done on up to date
   # files

   # Remove all old tree files in the current directory

   for file_name in glob.glob (tree_file_extension_filter):
      os.remove (file_name)

   # Build all tree files for the current project in the objects directory

   build_tree_files_command_line = (
      "gnatmake -c -gnatc -gnatt -P" +
      project_name)
   build_tree_files_process = GPS.Process (build_tree_files_command_line)
   build_tree_files_process.wait ()

   # Copy tree files from the objects directory to the current directory

   files_list = (
      glob.glob (objects_directory + "/" + tree_file_extension_filter))
      
   for file_name in files_list:
      shutil.copy (file_name, ".")

   # Run ASISEyes in the ASIS context created with tree files for the current
   # file. Note: command line argument using double quotes does not seems to
   # work, while single quote enclosed arguments are Ok.

   asiseyes_command_line = (
      "asiseyes-for-gps --file-name '" +
      current_ada_file_name +
      "'")

   messages_for_locations_window = []
   # Python is more efficient at contaneting multiple strings, using
   # an array of strings then join, rather than using the += operator.
   # Any output text which does not go there, goes to the message window
   # which follows.

   def on_asiseyes_process_terminated (process, exit_code, remaining_text):
   
      messages_for_locations_window.append (remaining_text)
      
      if exit_code == 0:

         # All three highlight categories set to an empty string to avoid
         # any highlight : near to all line of the file will match a location,
         # and the last thing we need, is to have a fully highlighted file.
         
         GPS.Locations.parse (
            output             = "".join (messages_for_locations_window),
            category           = "ASIS Interpretation",
            highlight_category = "",
            style_category     = "",
            warning_category   = "")
      else:
         message_window.write ("\nAn unknown error occured\n")

# All of this is drop, due to a GPS bug
#
#   message_window = GPS.Console (name = "ASISEyes")
#   # Create a message window. All text which is not to go to this window,
#   # will be added to messages_for_locations_window (no output is lost)
#
#   message_window.clear ()
#   # If another run was done, previous messages looks like garbage, so
#   # first clear the message window.
#
#   def on_asis_view_item_found (process, asis_item, other_message):
#
#      messages_for_locations_window.append (asis_item)
#      messages_for_locations_window.append ("\n")
#
#      if other_message.strip() != "":
#
#         # We will have to strip off ASIS-for-GNAT's warnings messages.
#         # These are not meaningful, at least for our purpose. To do this,
#         # we have to compare each lines of the message text.
#
#         message_lines = other_message.split ("\n")
#
#         for line in message_lines:
#            if line.find ("ASIS warning:") == 0:
#               None
#               # Drop this line
#            elif line.find ("Asis.Ada_Environments.Open:") == 0:
#               None
#               # Drop this line
#            else:
#               message_window.write (line)
#               message_window.write ("\n")
#
#   standard_gps_file_location_regexp = "^[^[:cntrl:]]+:[^[:cntrl:]]+:[^[:cntrl:]]+:[^[:cntrl:]]+$"
#
#   asiseyes_process = GPS.Process (
#      command            = asiseyes_command_line,
#      regexp             = standard_gps_file_location_regexp,
#      single_line_regexp = False,
#      on_match           = on_asis_view_item_found,
#      on_exit            = on_asiseyes_process_terminated)

# regexp = "^[\x20-\xFF]+:[\x20-\xFF]+:[\x20-\xFF]+:[\x20-\xFF]*$"
# The latter regexp makes GPS freeze

   asiseyes_process = GPS.Process (
      command = asiseyes_command_line,
      on_exit = on_asiseyes_process_terminated)

   asiseyes_process.wait ()

   # Clean the place
   for file_name in glob.glob (tree_file_extension_filter):
      os.remove (file_name)

# ============================================================================
# GPS event handlers (hooks)
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# When project properties changed, updates Ada file name extensions
# Hook: "project_view_changed"
# Updates global variables used by is_an_ada_file
# on_GPS_context_changed depends on is_an_ada_file, thus
# proper handling in on_GPS_context_changed depends on this one
def on_project_properties_changed (hook_name):

   project = GPS.Project.root()
   
   naming_package_name               = "Naming"
   ada_language_name                 = "Ada"
   spec_extension_attribute_name     = "Spec_Suffix"
   body_extension_attribute_name     = "Body_Suffix"
   separate_extension_attribute_name = "Separate_Suffix"

   global ada_spec_extension
   global ada_body_extension
   global ada_separate_extension

   ada_spec_extension = project.get_attribute_as_string (
      package   = naming_package_name,
      index     = ada_language_name,
      attribute = spec_extension_attribute_name)
      
   ada_body_extension = project.get_attribute_as_string (
      package   = naming_package_name,
      index     = ada_language_name,
      attribute = body_extension_attribute_name)
      
   ada_separate_extension = project.get_attribute_as_string (
      package   = naming_package_name,
      index     = ada_language_name,
      attribute = separate_extension_attribute_name)

# ----------------------------------------------------------------------------
# When context changed, enable/disable ASISEyes depending on context and
# save the current file name to be later use as ASISEyes parameter;
# Hook: "context_changed"
# This methods depends on Ada file name extensions
def on_GPS_context_changed (hook_name, context):
   context_class_name = context.__class__.__name__
   # The class name is used to determine the kind of context
   is_a_file_context = (
      (context_class_name == "FileContext") or
      (context_class_name == "AreaContext") or
      (context_class_name == "EntityContext"))
   # AreaContext is derived from FileContext.
   # EntityContext is derived from FileContext.
   if is_a_file_context:
      if is_an_ada_file (context.file().name()):
         global current_ada_file_name
         current_ada_file_name = context.file().name()
         enable_ASISEyes ()
      else:
         disable_ASISEyes ()
   else:
      disable_ASISEyes ()
   
# ----------------------------------------------------------------------------
# When the GPS UI is ready, setup add ASISEyes UI commands
# Hook: "gps_started"
def on_GPS_UI_ready (hook_name):
   # Better to use a Python script instead of a GPS XML configuration file,
   # because Python script allow much more flexibility than GPS configuration
   # files.
   
   # Set up the tool bar button.
   # This requires pyGTK to be present in order GUI.pywidget to work.
   # We first create a GPS button with a dummy label (dummy, because it will
   # later be overidden). Then get the associated GTK widget which will allow
   # to set image and tool-tip.
   
   guid = "{DF276442-3ADE-4f55-ACD8-9D33D1424E00}"
   toolbar = GPS.Toolbar ()
   GPS_button = GPS.Button (guid, "Dummy Label", on_click_run_asiseyes)
   GPS.Toolbar().append (GPS_button)
   GTK_button = GPS_button.pywidget ()
   GTK_button.remove (GTK_button.get_child ())
   gtk.Tooltips().set_tip(GTK_button, "Run ASIS Eyes")
   button_image_path = (
      GPS.get_system_dir() +
      "share/gps/plug-ins/asiseyes.png")
   image = gtk.Image ()
   image.set_from_file(button_image_path)
   GTK_button.add (image)
   image.show ()
   
   global ASISEyes_toolbar_button
   ASISEyes_toolbar_button = GTK_button

   # Set a menu entry in the Tools menu
   
   global ASISEyes_menu_item
   ASISEyes_menu_item = GPS.Menu.create ("/Tools/ASIS Eyes")

   disable_ASISEyes ()
   
# ============================================================================
# Setup
# ----------------------------------------------------------------------------
GPS.Hook ("project_view_changed").add (on_project_properties_changed)
GPS.Hook ("context_changed").add (on_GPS_context_changed)
GPS.Hook ("gps_started").add (on_GPS_UI_ready)
