
--! Do not modify this pattern, use a copy with a meaningful name instead.
--! See the corresponding implementation pattern to know how you should
--! update.

-- This package is the reference specification for a static genericity.
-- This package provides a default concret and working implementation, but
-- still have a look at its sibling packages.
--
-- See My_Asis_Vendor to switch from one to another alternative.
--
-- Provides implementation of one formally Asis implementation defined
-- functionality : start an Asis environment. Provides implementation of a
-- utility functionality which is not part of Asis : mapping from file name
-- to unit name.

with Asis;

package ASIS_Vendors.Typical is

   procedure Begin_With_Context
     (Context : out Asis.Context; Name : Wide_String);
   -- Due to implementation defined Parameters in an Asis call, this method is
   -- implementation specific, while a concret default implementation is
   -- possible. Alternative implementations should mainly update this method
   -- implementation (with Unit_Name_From_File_Name, if applicable).

   procedure End_With_Context
     (Context : in out Asis.Context);
   -- Unlike Begin_With_Context, this method is not implementation specific,
   -- but it is there for consistency : better to have a Close/End methods
   -- in the same package to which the Open/Begin method belongs. Alternative
   -- implementations should use exactly the same one as provided in this
   -- default implementation.

   function Unit_Name_From_File_Name
     (Context   : in Asis.Context;
      File_Name : in Wide_String)
     return Wide_String;
   --|Requires: File_Name'Length /= 0
   -- Returns Not_A_Unit_Name is not found.
   -- Note: the specification must be a function returning a string, as the
   -- client may not priorly know the size to give to string used as an
   -- out parameter. To have an out string and an out boolean, would have
   -- been nice, but it is not feasable.
   -- Note: multiple file names may match a same unit name.
   -- Note: although Asis provides request to get the file name associated
   -- to unit, the Ada specification has no requirement at all for an Ada
   -- compiler/interpreter to rely on file to store Ada source text. So
   -- this method may makes no sense in some environments.

end ASIS_Vendors.Typical;
