
-- To build this package for an ASIS vendor of your choice, update the
-- renaming package My_ASIS_Vendor.

-- To build this package to target an IDE vendor of your choice, update the
-- renaming package My_IDE_Vendor.

with ASIS_Vendors;
with My_ASIS_Vendor;

with IDE_Vendors;
with My_IDE_Vendor;

with Asis.Implementation;
with Asis.Ada_Environments;
with Asis.Compilation_Units;
with Asis.Elements;
with Asis.Declarations;
with Asis.Iterator;
with Asis.Statements;
with Asis.Expressions;
with Asis.Text;

with Tiny.Texts;
with Ada.Wide_Text_IO;

package body ASISEyes is

   -- ------------------------------------------------------------------------

   Lower_Level  : constant := 1;
   Higher_Level : constant := 128;
   Context_Name : constant Wide_String := "Single_Context";

   Indent : constant Wide_String := "   ";

   Open_Entity_Name   : constant Wide_String := "<";
   Close_Entity_Name  : constant Wide_String := ">";
   Prefix_To_Selector : constant Wide_String := ".";

   Key_To_Value                     : constant Wide_String :=
     (" : ");
   Full_Name_Key                    : constant Wide_String :=
     ("Full name");
   File_Name_Key                    : constant Wide_String :=
     ("File name");
   Unit_Kind_Key                    : constant Wide_String :=
     ("Unit kind");
   Unit_Class_Key                   : constant Wide_String :=
     ("Unit class");
   Unit_Origin_Key                  : constant Wide_String :=
     ("Unit origin");
   Unit_Name_Key                    : constant Wide_String :=
     ("Unit name");
   ASIS_Vesion_Key                  : constant Wide_String :=
     ("ASIS Version");
   ASIS_Implementor_Key             : constant Wide_String :=
     ("ASIS Implementator");
   ASIS_Implementor_Version_Key     : constant Wide_String :=
     ("ASIS Implementor Version");
   ASIS_Implementor_Information_Key : constant Wide_String :=
     ("ASIS Implementor Information");
   ASIS_Default_Context_Name_Key    : constant Wide_String :=
     ("ASIS Default Context Name");
   ASIS_Default_Parameters_Key      : constant Wide_String :=
     ("ASIS Default Parameters");

   Unit_Specification_Reference_Title    : constant Wide_String :=
     ("Reference for unit specification");
   Unit_Body_Reference_Title             : constant Wide_String :=
     ("Reference for unit body");
   Unit_Specification_Declarations_Title : constant Wide_String :=
     ("Declarations in unit specification");
   Unit_Body_Declarations_Title          : constant Wide_String :=
     ("Declarations in unit specification");
   Searching_Unit_From_File_Name_Title   : constant Wide_String :=
     ("Searching for the unit which match the file name");

   Tag_To_Content         : constant Wide_String := " ";
   Content_Item_Separator : constant Wide_String := " ";
   Pragma_Tag             : constant Wide_String := "pragma";
   Declaration_Tag        : constant Wide_String := "declare";
   Definition_Tag         : constant Wide_String := "definition";
   Expression_Tag         : constant Wide_String := "expression";
   Association_Tag        : constant Wide_String := "association";
   Statement_Tag          : constant Wide_String := "statement";
   Path_Tag               : constant Wide_String := "path";
   Clause_Tag             : constant Wide_String := "clause";
   Exception_Handler_Tag  : constant Wide_String := "exception handler";
   Others_Choice_Tag      : constant Wide_String := "others";

   Not_Found_Indicator                           : constant Wide_String :=
     ("Not Found");
   No_Unit_Matching_File_Name_Indicator          : constant Wide_String :=
     ("No unit found matching the file name");
   Found_Indicator                               : constant Wide_String :=
     ("Found");
   Unit_Not_Found_Indicator                      : constant Wide_String :=
     ("Unit not found");
   Invalid_Exception_Choice_Indicator            : constant Wide_String :=
     ("* ERROR * Invalid exception choice kind");
   Invalid_Element_Kind_Indicator                : constant Wide_String :=
     ("* ERROR * Invalid element");
   Invalid_Exception_Choice_Definition_Indicator : constant Wide_String :=
     ("* ERROR * Invalid exception choice definition kind");
   Invalid_Exception_Choice_Expression_Indicator : constant Wide_String :=
     ("* ERROR * Invalid exception choice expression kind");

   -- ------------------------------------------------------------------------

   -- In order to avoid confusion between ASIS.Text and Tiny.Texts, the
   -- Texts prefix is not used in this text source and what is needed
   -- from Texts, is imported here.

   ASCII_Replacement_Character : constant Character :=
     Tiny.Texts.ASCII_Replacement_Character;

   function To_String
     (Wide                  : Wide_String;
      Replacement_Character : Character := ASCII_Replacement_Character)
      return String
      renames Tiny.Texts.To_String;

   -- ------------------------------------------------------------------------

   procedure Log_Message (Message : in Wide_String)
   is
   begin
      My_IDE_Vendor.Log_Message (To_String (Message));
   end Log_Message;

   -- ------------------------------------------------------------------------

   procedure End_Of_Paragraph
   is
   begin
      My_IDE_Vendor.End_Of_Paragraph;
   end End_Of_Paragraph;

   -- ------------------------------------------------------------------------

   procedure Put_ASIS_Configuration
   is
      package Implementation renames Asis.Implementation;
      package Ada_Environments renames Asis.Ada_Environments;
   begin
      Log_Message
        (ASIS_Vesion_Key &
         Key_To_Value &
         Implementation.ASIS_Version);

      Log_Message
        (ASIS_Implementor_Key &
         Key_To_Value &
         Implementation.ASIS_Implementor);

      Log_Message
        (ASIS_Implementor_Version_Key &
         Key_To_Value &
         Implementation.ASIS_Implementor_Version);

      Log_Message
        (ASIS_Implementor_Information_Key &
         Key_To_Value &
         Implementation.ASIS_Implementor_Information);

      Log_Message
        (ASIS_Default_Context_Name_Key &
         Key_To_Value &
         Ada_Environments.Default_Name);

      Log_Message
        (ASIS_Default_Parameters_Key &
         Key_To_Value &
         Ada_Environments.Default_Parameters);

      End_Of_Paragraph;

   end Put_ASIS_Configuration;

   -- ------------------------------------------------------------------------

   function Unit_Exist
     (Context   : in Asis.Context;
      Unit_Name : in Wide_String)
      return Boolean
   is
      package Compilation_Units renames Asis.Compilation_Units;
   begin
      Test_As_Unit_Declaration :
      declare
         Library_Unit : constant Asis.Compilation_Unit :=
           Compilation_Units.Library_Unit_Declaration
             (Name        => Unit_Name,
              The_Context => Context);
      begin
         if not Compilation_Units.Is_Nil (Library_Unit) then
            return True;
         end if;
      end Test_As_Unit_Declaration;
      Test_As_Unit_Body :
      declare
         Body_Unit : constant Asis.Compilation_Unit :=
           Compilation_Units.Compilation_Unit_Body
             (Name        => Unit_Name,
              The_Context => Context);
      begin
         if not Compilation_Units.Is_Nil (Body_Unit) then
            return True;
         end if;
      end Test_As_Unit_Body;
      return False;
   end Unit_Exist;

   -- ------------------------------------------------------------------------

   procedure Put_Compilation_Unit_Reference (Unit : Asis.Compilation_Unit)
   -- Used by Put_Unit_Specification_Reference and Put_Unit_Body_Reference
   is
      package Compilation_Units renames Asis.Compilation_Units;
   begin
      if not Compilation_Units.Is_Nil (Unit) then
         Existing_Unit :
         declare
            No_File_Name : constant Wide_String :=
              ("");
            Full_Name    : constant Wide_String :=
              (Compilation_Units.Unit_Full_Name (Unit));
            File_Name    : constant Wide_String :=
              (Compilation_Units.Text_Name (Unit));
            Kind         : constant Asis.Unit_Kinds :=
              (Compilation_Units.Unit_Kind (Unit));
            Cls          : constant Asis.Unit_Classes :=
              (Compilation_Units.Unit_Class (Unit));
            Origin       : constant Asis.Unit_Origins :=
              (Compilation_Units.Unit_Origin (Unit));
         begin

            Log_Message (Indent & Full_Name_Key & Key_To_Value & Full_Name);

            if File_Name /= No_File_Name then
               Log_Message
                 (Indent & File_Name_Key & Key_To_Value & File_Name);
            end if;

            Log_Message
              (Indent &
               Unit_Kind_Key &
               Key_To_Value &
               Asis.Unit_Kinds'Wide_Image (Kind));

            Log_Message
              (Indent &
               Unit_Class_Key &
               Key_To_Value &
               Asis.Unit_Classes'Wide_Image (Cls));

            Log_Message
              (Indent &
               Unit_Origin_Key &
               Key_To_Value &
               Asis.Unit_Origins'Wide_Image (Origin));

         end Existing_Unit;
      else
         Log_Message (Indent & Not_Found_Indicator);
      end if;
      End_Of_Paragraph;
   end Put_Compilation_Unit_Reference;

   -- ------------------------------------------------------------------------

   procedure Put_Unit_Specification_Properties
     (Context   : in Asis.Context;
      Unit_Name : in Wide_String)
   is
      package Compilation_Units renames Asis.Compilation_Units;
      Unit : constant Asis.Compilation_Unit :=
        Compilation_Units.Library_Unit_Declaration
          (Name        => Unit_Name,
           The_Context => Context);
   begin
      Log_Message
        (Unit_Specification_Reference_Title &
         Content_Item_Separator &
         Open_Entity_Name &
         Unit_Name &
         Close_Entity_Name);

      Put_Compilation_Unit_Reference (Unit);

   end Put_Unit_Specification_Properties;

   -- ------------------------------------------------------------------------

   procedure Put_Unit_Body_Properties
     (Context   : in Asis.Context;
      Unit_Name : in Wide_String)
   is
      package Compilation_Units renames Asis.Compilation_Units;
      Unit : constant Asis.Compilation_Unit :=
        Compilation_Units.Compilation_Unit_Body
          (Name        => Unit_Name,
           The_Context => Context);
   begin
      Log_Message
        (Unit_Body_Reference_Title &
         Content_Item_Separator &
         Open_Entity_Name &
         Unit_Name &
         Close_Entity_Name);

      Put_Compilation_Unit_Reference (Unit);

   end Put_Unit_Body_Properties;

   -- ------------------------------------------------------------------------

   procedure Put_Element_Summary (Element : in Asis.Element)
   is

      package IO           renames Ada.Wide_Text_IO;
      package Elements     renames Asis.Elements;
      package Declarations renames Asis.Declarations;
      package Statements   renames Asis.Statements;
      package Expressions  renames Asis.Expressions;

      -- The latter packages stands for what en element may actually be and
      -- are required to access more detailed informations about the element.

      Element_Kind : constant Asis.Element_Kinds :=
        Elements.Element_Kind (Element);
   begin
      case Element_Kind is
         when Asis.A_Pragma =>
            Put_Pragma_Kind :
            declare
               The_Pragma  : constant Asis.Pragma_Element := Element;
               Pragma_Kind : constant Asis.Pragma_Kinds :=
                 Elements.Pragma_Kind (The_Pragma);
               -- Most of what a pragma means, is said in Pragma_Kind.
               -- The remaining is parameters which are not shown here.
            begin
               IO.Put (Pragma_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Pragma_Kinds'Wide_Image (Pragma_Kind));
            end Put_Pragma_Kind;
         when Asis.A_Defining_Name =>
            Put_Name_Text_And_Kind :
            declare
               Defining_Name      : constant Asis.Defining_Name :=
                 Element;
               Defining_Name_Kind : constant Asis.Defining_Name_Kinds :=
                 Elements.Defining_Name_Kind (Defining_Name);
               -- If A_Defining_Operator_Symbol, then Operator_Kinds is
               -- anyway given implicitely with the following Name_Text.
               Name_Text          : constant Wide_String :=
                 Declarations.Defining_Name_Image
                   (Defining_Name);
            begin
               IO.Put
                 (Open_Entity_Name &
                  Name_Text &
                  Close_Entity_Name &
                  Content_Item_Separator &
                  Asis.Defining_Name_Kinds'Wide_Image (Defining_Name_Kind));
            end Put_Name_Text_And_Kind;
         when Asis.A_Declaration =>
            Put_Declaration_Kind_And_Trait :
            declare
               Declaration            : constant Asis.Declaration := Element;
               Declaration_Kind       : constant Asis.Declaration_Kinds :=
                 Elements.Declaration_Kind (Declaration);
               Declaration_Trait_Kind : constant Asis.Trait_Kinds :=
                 Elements.Trait_Kind (Declaration);
            begin
               IO.Put (Declaration_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Declaration_Kinds'Wide_Image (Declaration_Kind));
               case Declaration_Trait_Kind is
                  when Asis.Not_A_Trait =>
                     null;
                  when others =>
                     IO.Put
                       (Content_Item_Separator);
                     IO.Put
                       (Asis.Trait_Kinds'Wide_Image (Declaration_Trait_Kind));
               end case;
            end Put_Declaration_Kind_And_Trait;
         when Asis.A_Definition =>
            Put_Definition_Kind :
            declare
               Definition      : constant Asis.Definition := Element;
               Definition_Kind : constant Asis.Definition_Kinds :=
                 Elements.Definition_Kind (Definition);
            begin
               IO.Put (Definition_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Definition_Kinds'Wide_Image (Definition_Kind));
            end Put_Definition_Kind;
         when Asis.An_Expression =>
            Put_Expression_Kind :
            declare
               Expression      : constant Asis.Expression := Element;
               Expression_Kind : constant Asis.Expression_Kinds :=
                 Elements.Expression_Kind (Expression);
            begin
               IO.Put (Expression_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Expression_Kinds'Wide_Image (Expression_Kind));
            end Put_Expression_Kind;
         when Asis.An_Association =>
            Put_Association_Kind :
            declare
               Association      : constant Asis.Association := Element;
               Association_Kind : constant Asis.Association_Kinds :=
                 Elements.Association_Kind (Association);
            begin
               IO.Put (Association_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Association_Kinds'Wide_Image (Association_Kind));
            end Put_Association_Kind;
         when Asis.A_Statement =>
            Put_Statement_Kind :
            declare
               Statement      : constant Asis.Statement := Element;
               Statement_Kind : constant Asis.Statement_Kinds :=
                 Elements.Statement_Kind (Statement);
            begin
               IO.Put (Statement_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Statement_Kinds'Wide_Image (Statement_Kind));
            end Put_Statement_Kind;
         when Asis.A_Path =>
            Put_Path_Kind :
            declare
               Path      : constant Asis.Path := Element;
               Path_Kind : constant Asis.Path_Kinds :=
                 Elements.Path_Kind (Path);
            begin
               IO.Put (Path_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Path_Kinds'Wide_Image (Path_Kind));
            end Put_Path_Kind;
         when Asis.A_Clause =>
            Put_Clause_Kind_And_Optionaly_Representation_Kind :
            declare
               Clause : constant Asis.Clause := Element;
               Clause_Kind : constant Asis.Clause_Kinds :=
                 Elements.Clause_Kind (Clause);
               Representation_Clause_Kind : constant
                 Asis.Representation_Clause_Kinds :=
                   Elements.Representation_Clause_Kind (Clause);
            begin
               IO.Put (Clause_Tag);
               IO.Put (Tag_To_Content);
               IO.Put (Asis.Clause_Kinds'Wide_Image (Clause_Kind));
               case Representation_Clause_Kind is
                  when Asis.Not_A_Representation_Clause =>
                     null;
                  when others =>
                     IO.Put (Content_Item_Separator);
                     IO.Put
                       (Asis.Representation_Clause_Kinds'Wide_Image
                          (Representation_Clause_Kind));
               end case;
            end Put_Clause_Kind_And_Optionaly_Representation_Kind;
         when Asis.An_Exception_Handler =>
            Put_Handler_Choices_List :
            declare
               Handler : constant Asis.Exception_Handler := Element;
               Choices : constant Asis.Element_List :=
                 Statements.Exception_Choices (Handler);
            begin
               IO.Put (Exception_Handler_Tag);
               IO.Put (Tag_To_Content);
               for I in Choices'Range loop
                  Put_Choice_Expression_Or_Definition :
                  declare
                     Choice      : constant Asis.Element := Choices (I);
                     Choice_Kind : constant Asis.Element_Kinds :=
                       Elements.Element_Kind (Choice);
                  begin
                     case Choice_Kind is
                        when Asis.An_Expression =>
                           Put_Exception_Choice_Identifier_Component_Name :
                           declare
                              Expression : constant Asis.Expression := Choice;
                              Expression_Kind       : constant
                                Asis.Expression_Kinds :=
                                  Elements.Expression_Kind (Expression);
                           begin
                              case Expression_Kind is
                                 when Asis.An_Identifier =>
                                    Put_Exception_Choice_Identifier_Name :
                                    declare
                                       Identifier : constant
                                         Asis.Identifier :=
                                           Expression;
                                       Identifier_Name : constant
                                         Asis.Program_Text :=
                                           Expressions.Name_Image
                                             (Identifier);
                                    begin
                                       IO.Put (Identifier_Name);
                                    end Put_Exception_Choice_Identifier_Name;
                                 when Asis.A_Selected_Component =>
                                    Put_Exception_Choice_Selector :
                                    declare
                                       Selected_Component : constant
                                         Asis.Expression :=
                                           Expression;
                                       Prefix : constant Asis.Expression :=
                                         Expressions.Prefix
                                           (Selected_Component);
                                       Prefix_Kind : constant
                                         Asis.Expression_Kinds :=
                                           Elements.Expression_Kind (Prefix);
                                       Selector : constant Asis.Expression :=
                                         Expressions.Selector
                                           (Selected_Component);
                                       Selector_Name : constant
                                         Asis.Program_Text :=
                                           Expressions.Name_Image (Selector);
                                    begin
                                       IO.Put
                                         (Asis.Expression_Kinds'Wide_Image
                                            (Prefix_Kind));
                                       IO.Put
                                         (Prefix_To_Selector);
                                       IO.Put
                                         (Selector_Name);
                                    end Put_Exception_Choice_Selector;
                                 when others =>
                                    IO.Put
                                      (Invalid_Exception_Choice_Expression_Indicator);
                              end case;
                           end Put_Exception_Choice_Identifier_Component_Name;
                        when Asis.A_Definition =>
                           Put_Exception_Choice_Definition_Kind :
                           declare
                              Definition : constant Asis.Definition := Choice;
                              Definition_Kind : constant
                                Asis.Definition_Kinds :=
                                  Elements.Definition_Kind (Definition);
                           begin
                              case Definition_Kind is
                                 when Asis.An_Others_Choice =>
                                    IO.Put (Others_Choice_Tag);
                                 when others =>
                                    IO.Put
                                      (Invalid_Exception_Choice_Definition_Indicator);
                              end case;
                           end Put_Exception_Choice_Definition_Kind;
                        when others =>
                           IO.Put (Invalid_Exception_Choice_Indicator);
                     end case;
                  end Put_Choice_Expression_Or_Definition;
                  if I < Choices'Last then
                     IO.Put (Content_Item_Separator);
                  end if;
               end loop;
            end Put_Handler_Choices_List;
         when Asis.Not_An_Element =>
            IO.Put (Invalid_Element_Kind_Indicator);
      end case;
   end Put_Element_Summary;

   -- ------------------------------------------------------------------------

   procedure Put_Declarations (Unit : Asis.Compilation_Unit)
   is
      package Compilation_Units renames Asis.Compilation_Units;
      package Elements          renames Asis.Elements;
      package Iterators         renames Asis.Iterator;
      package Text              renames Asis.Text;
   begin

      Special_Cases :
      begin
         if Compilation_Units.Is_Nil (Unit) then
            Log_Message (Indent & Unit_Not_Found_Indicator);
            End_Of_Paragraph;
            return;
         end if;
      end Special_Cases;

      Common_Case :
      declare

         type Level_Type is new Positive range Lower_Level .. Higher_Level;

         File_Name : aliased constant String :=
           To_String (Compilation_Units.Text_Name (Unit));

         procedure Log_Element_Summary_With_Ident_And_Position
           (Element : in Asis.Element;
            Level   : in Level_Type)
         is
            type Element_Summy_To_Write_Type is
              new IDE_Vendors.Text_To_Write_Type with
                null record;

            procedure Write (Text_To_Write : Element_Summy_To_Write_Type)
            is
               procedure Put_Indent (Level : in Level_Type)
               is
                  package IO renames Ada.Wide_Text_IO;
               begin
                  for I in Level_Type'Succ (Level_Type'First) .. Level loop
                     IO.Put (Indent);
                  end loop;
               end Put_Indent;
            begin
               pragma Unreferenced (Text_To_Write);
               -- Only use out Element and Level parameters
               Put_Indent (Level);
               Put_Element_Summary (Element);
            end Write;

            Element_Summy_To_Write : Element_Summy_To_Write_Type;

            Span : constant Text.Span := Text.Element_Span (Element);
            -- Used to gey Line_Number and Column_Number

            Line_Number   : constant Positive := Span.First_Line;
            Column_Number : constant Positive := Span.First_Column;
         begin
            My_IDE_Vendor.Log_Message_With_File_Position
              (File_Name     => File_Name'Access,
               Line_Number   => Line_Number,
               Column_Number => Column_Number,
               Log_Text      => Element_Summy_To_Write);

         end Log_Element_Summary_With_Ident_And_Position;

         procedure Put_Unit_Context_Clauses (Unit : Asis.Compilation_Unit) is
            Context_Clauses : constant Asis.Context_Clause_List :=
              Elements.Context_Clause_Elements
                (Compilation_Unit => Unit,
                 Include_Pragmas  => True);
         begin
            for I in Context_Clauses'Range loop
               declare
                  Clause : constant Asis.Clause := Context_Clauses (I);
               begin
                  Log_Element_Summary_With_Ident_And_Position
                    (Element => Clause,
                     Level   => Level_Type'First);
               end;
            end loop;
         end Put_Unit_Context_Clauses;

         procedure When_Enter_In_Element
           (Element : in Asis.Element;
            Control : in out Asis.Traverse_Control;
            Level   : in out Level_Type)
         is
         begin
            Log_Element_Summary_With_Ident_And_Position
              (Element => Element,
               Level   => Level);

            if Level < Level_Type'Last then
               Control := Asis.Continue;
               Level := Level_Type'Succ (Level);
            else
               Control := Asis.Abandon_Children;
            end if;
         end When_Enter_In_Element;

         procedure When_Exit_From_Element
           (Element : in Asis.Element;
            Control : in out Asis.Traverse_Control;
            Level   : in out Level_Type) is
         begin
            pragma Unreferenced (Element);
            pragma Unreferenced (Control);
            if Level > Level_Type'First then
               Level := Level_Type'Pred (Level);
            end if;
         end When_Exit_From_Element;

         procedure Iterate
           (Root          : in Asis.Element;
            Control       : in out Asis.Traverse_Control;
            Initial_Level : in out Level_Type)
         is
            procedure The_Implementation is
              new Iterators.Traverse_Element
                (State_Information => Level_Type,
                 Pre_Operation     => When_Enter_In_Element,
                 Post_Operation    => When_Exit_From_Element);
         begin
            The_Implementation
              (Element => Root,
               Control => Control,
               State   => Initial_Level);
         end Iterate;

         procedure Iterate (Root : in Asis.Element)
         is
            Control       : Asis.Traverse_Control := Asis.Continue;
            Initial_Level : Level_Type := Level_Type'First;
         begin
            Iterate
              (Root          => Root,
               Control       => Control,
               Initial_Level => Initial_Level);
         end Iterate;

      begin

         Put_Unit_Context_Clauses (Unit);

         declare
            Root_Element : constant Asis.Element :=
              Elements.Unit_Declaration (Unit);
         begin
            Iterate (Root_Element);
         end;
      end Common_Case;
   end Put_Declarations;

   -- ------------------------------------------------------------------------

   procedure Put_Unit_Specification_Declarations
     (Context   : in Asis.Context;
      Unit_Name : in Wide_String)
   is
      package Compilation_Units renames Asis.Compilation_Units;
      Unit : constant Asis.Compilation_Unit :=
        Compilation_Units.Library_Unit_Declaration
          (Name        => Unit_Name,
           The_Context => Context);
   begin
      Log_Message
        (Unit_Specification_Declarations_Title &
         Content_Item_Separator &
         Open_Entity_Name &
         Unit_Name &
         Close_Entity_Name);
      Put_Declarations (Unit);
   end Put_Unit_Specification_Declarations;

   -- ------------------------------------------------------------------------

   procedure Put_Unit_Body_Declarations
     (Context   : in Asis.Context;
      Unit_Name : in Wide_String)
   is
      package Compilation_Units renames Asis.Compilation_Units;
      Unit : constant Asis.Compilation_Unit :=
        Compilation_Units.Compilation_Unit_Body
          (Name        => Unit_Name,
           The_Context => Context);
   begin
      Log_Message
        (Unit_Body_Declarations_Title &
         Content_Item_Separator &
         Open_Entity_Name &
         Unit_Name &
         Close_Entity_Name);
      Put_Declarations (Unit);
   end Put_Unit_Body_Declarations;

   -- ------------------------------------------------------------------------

   procedure ASISEyes
     (Context   : in Asis.Context;
      Unit_Name : in Wide_String)
   --|Requires: Asis.Ada_Environments.Is_Open (Context)
   is
   begin
      Put_Unit_Specification_Properties
        (Context   => Context,
         Unit_Name => Unit_Name);

      Put_Unit_Body_Properties
        (Context   => Context,
         Unit_Name => Unit_Name);

      Put_Unit_Specification_Declarations
        (Context   => Context,
         Unit_Name => Unit_Name);

      Put_Unit_Body_Declarations
        (Context   => Context,
         Unit_Name => Unit_Name);
   end ASISEyes;

   -- ------------------------------------------------------------------------

   procedure ASISEyes_Using_Unit_Name (Unit_Name : in Wide_String)
   is
      Context : Asis.Context;
   begin
      Put_ASIS_Configuration;

      My_ASIS_Vendor.Begin_With_Context
        (Context => Context,
         Name    => Context_Name);

      ASISEyes
        (Context   => Context,
         Unit_Name => Unit_Name);

      My_ASIS_Vendor.End_With_Context (Context);

   end ASISEyes_Using_Unit_Name;

   -- ------------------------------------------------------------------------

   procedure ASISEyes_Using_File_Name (File_Name : in Wide_String)
   is
      Context : Asis.Context;
   begin
      Put_ASIS_Configuration;

      My_ASIS_Vendor.Begin_With_Context
        (Context => Context,
         Name    => Context_Name);

      Process_Inside_ASIS_Context :
      begin
         Log_Message (Searching_Unit_From_File_Name_Title);
         Log_Message (Indent & File_Name_Key & Key_To_Value & File_Name);
         Call_ASISEyes_Getting_Name_From_File_Name :
         declare
            Unit_Name : constant Wide_String :=
              My_ASIS_Vendor.Unit_Name_From_File_Name
                (Context   => Context,
                 File_Name => File_Name);
         begin
            if Unit_Name = ASIS_Vendors.Not_A_Unit_Name then
               Log_Message (Indent & No_Unit_Matching_File_Name_Indicator);
               End_Of_Paragraph;
            else
               Log_Message
                 (Indent &
                  Found_Indicator);

               Log_Message
                 (Indent &
                  Unit_Name_Key &
                  Key_To_Value &
                  Unit_Name);

               End_Of_Paragraph;

               ASISEyes
                 (Context   => Context,
                  Unit_Name => Unit_Name);
            end if;
         end Call_ASISEyes_Getting_Name_From_File_Name;
      end Process_Inside_ASIS_Context;

      My_ASIS_Vendor.End_With_Context (Context);

   end ASISEyes_Using_File_Name;

end ASISEyes;
