
--! Do not modify this pattern, use a copy with a meaningful name instead.
--! See the corresponding implementation pattern to know how you should
--! update.

-- This package is the reference specification for a static genericity.
-- This package provides a default concret and working implementation, but
-- still have a look at its sibling packages.
--
-- See My_IDE_Vendor to switch from one to another alternative.
--
-- This provide outputs formated in a way expected by IDEs for their
-- external tools.

package IDE_Vendors.Typical is

   -- ========================================================================
   -- Table of Content
   -- ------------------------------------------------------------------------
   -- 1. End of paraggraph
   -- 2. Raw messages
   -- 3. Message associated to a file/text location
   -- ------------------------------------------------------------------------

   -- ========================================================================
   -- 1. End of paraggraph
   -- ------------------------------------------------------------------------

   procedure End_Of_Paragraph;

   -- ========================================================================
   -- 2. Raw messages
   -- ------------------------------------------------------------------------

   procedure Log_Message (Log_Text : in String);
   -- The most handy

   procedure Log_Message (Log_Text : access constant String);
   -- Less handy but more efficient

   procedure Log_Message (Log_Text : in out Text_To_Write_Class);
   -- "Not handy", but really efficient

   -- ========================================================================
   -- 3. Message associated to a file/text location
   -- ------------------------------------------------------------------------

   -- Most of times, the file name is common to many message of an external
   -- tool outputs, so it is very likely it will stored somewhere in the
   -- application and not computed for each message output. This is the
   -- reason why it is passed as an access to constant string in the three
   -- methods.

   procedure Log_Message_With_File_Position
     (File_Name     : access constant String;
      Line_Number   : in Positive;
      Column_Number : in Positive;
      Log_Text      : in String);
   -- The most handy

   procedure Log_Message_With_File_Position
     (File_Name     : access constant String;
      Line_Number   : in Positive;
      Column_Number : in Positive;
      Log_Text      : access constant String);
   -- Less handy but more efficient

   procedure Log_Message_With_File_Position
     (File_Name     : access constant String;
      Line_Number   : in Positive;
      Column_Number : in Positive;
      Log_Text      : in out Text_To_Write_Class);
   -- "Not handy", but really efficient

end IDE_Vendors.Typical;
