
-- Root package of a set of child packages implementing concret interface
-- to IDE expected external tools output.

-- See IDE_Vendors.Typical for the reference specification of these packages.

package IDE_Vendors is

   type Text_To_Write_Type is abstract tagged private;
   -- A text abstraction when a text/string is expected as a parameter,
   -- and this text/string is intended to be written on an output stream
   -- only. Using an abstraction, avoid to have to transport what is not
   -- formally required, that is, the text/string, and help improving
   -- performances, avoiding the use of unconstrained type, memory allocation
   -- and the like. Formally speaking, such a design strategy should place
   -- these kinds of abstracted data+operation types in packages defining
   -- operations using the type as arguments. Thus, this Text_To_Write_Type
   -- would be better placed in Ada.Text_IO. But the latter is a predefined
   -- package which must not or cannot be modified. Some applications may
   -- prefer this abstract type instead of access to allocated string
   -- storage or unconstrained string. Thus, IDE_Vendors child packagse are
   -- required to implement methods which use this abstract type as a
   -- parameters beside the equivalent methods which use more classic string
   -- parameters. See IDE_Vendors.Typical for the reference specification of
   -- the methods to be implemented.

   subtype Text_To_Write_Class is Text_To_Write_Type'Class;
   -- Type to use for parameters of actual type Text_To_Write_Type.

   procedure Write (Text_To_Write : Text_To_Write_Type) is abstract;
   -- The usage of the abstract type which makes sense over any instance
   -- of the type it represent (here, a String text).

   -- ========================================================================
   -- ***** DO NOT CROSS ***** DO NOT CROSS ***** DO NOT CROSS ***** DO NOT CR
   -- ========================================================================

private

   type Text_To_Write_Type is abstract tagged null record;

end IDE_Vendors;
