
with Tiny.Containers;

pragma Elaborate_All (Tiny.Containers);

package body Shell_UI.Application_Description is

   -- ========================================================================
   -- Table of Content
   -- ------------------------------------------------------------------------
   -- 1. Application name and purpose
   -- 2. Application copyrights
   -- 3. Application version
   -- ------------------------------------------------------------------------

   -- ========================================================================
   -- 1. Application name and purpose
   -- ------------------------------------------------------------------------

   Name_Access : access String := null;

   -- ------------------------------------------------------------------------

   Purpose_Text_Access : access String := null;

   -- ------------------------------------------------------------------------

   function Name_Fixed return Boolean
   is
   begin
      return Name_Access /= null;
   end Name_Fixed;

   -- ------------------------------------------------------------------------

   function Purpose_Text_Fixed return Boolean
   is
   begin
      return Purpose_Text_Access /= null;
   end Purpose_Text_Fixed;

   -- ------------------------------------------------------------------------

   function Name return String
   is
   begin
      if not Name_Fixed then
         raise Program_Error;
      end if;

      return Name_Access.all;
   end Name;

   -- ------------------------------------------------------------------------

   function Purpose_Text return String
   is
      Undefined_Purpose : constant String := "";
   begin
      if not Purpose_Text_Fixed then
         return Undefined_Purpose;
      end if;

      return Purpose_Text_Access.all;
   end Purpose_Text;

   -- ------------------------------------------------------------------------

   procedure Set_Name (Application_Name : in String)
   is
   begin
      if Name_Fixed then
         raise Program_Error;
      end if;

      Name_Access := new String'(Application_Name);
   end Set_Name;

   -- ------------------------------------------------------------------------

   procedure Set_Purpose_Text (Text : in String)
   is
   begin
      if Purpose_Text_Fixed then
         raise Program_Error;
      end if;

      Purpose_Text_Access := new String'(Text);
   end Set_Purpose_Text;

   -- ========================================================================
   -- 2. Application copyrights
   -- ------------------------------------------------------------------------
   -- 2.1 Copyrights datas
   -- 2.2 Copyrights set/get
   -- 2.3 Iterator on third parties copyright
   -- ------------------------------------------------------------------------

   -- ========================================================================
   -- 2.1 Copyrights datas
   -- ------------------------------------------------------------------------

   Main_Copyright_Notice_Access : access String := null;

   -- ------------------------------------------------------------------------

   type Third_Party_Copyright_Type is record
      Component_Name   : access constant String;
      Copyright_Notice : access constant String;
   end record;

   package Copyrights_Lists is new Tiny.Containers.Vectors
     (Element_Type => Third_Party_Copyright_Type);

   -- ------------------------------------------------------------------------

   Third_Parties_Copyrights : Copyrights_Lists.Vector_Access :=
     (Copyrights_Lists.Null_Vector);

   function Main_Copyright_Notice_Fixed return Boolean
   is
   begin
      return Main_Copyright_Notice_Access /= null;
   end Main_Copyright_Notice_Fixed;

   -- ------------------------------------------------------------------------

   function There_Are_Third_Parties_Copyrights return Boolean
   is
   begin
      return not Copyrights_Lists.Is_Null (Third_Parties_Copyrights);
   end There_Are_Third_Parties_Copyrights;

   -- ========================================================================
   -- 2.2 Copyrights set/get
   -- ------------------------------------------------------------------------

   function Main_Copyright_Notice return String
   is
      Undefined_Copyright : constant String := "";
   begin
      if not Main_Copyright_Notice_Fixed then
         return Undefined_Copyright;
      end if;

      return Main_Copyright_Notice_Access.all;
   end Main_Copyright_Notice;

   -- ------------------------------------------------------------------------

   procedure Set_Main_Copyright_Notice (Copyright_Notice : in String)
   is
   begin
      if Main_Copyright_Notice_Fixed then
         raise Program_Error;
      end if;

      Main_Copyright_Notice_Access := new String'(Copyright_Notice);
   end Set_Main_Copyright_Notice;

   -- ------------------------------------------------------------------------

   procedure Add_Third_Party_Copyright_Notice
     (Component_Name   : in String;
      Copyright_Notice : in String)
   is
   begin
      if not Main_Copyright_Notice_Fixed then
         raise Program_Error;
      end if;

      declare
         New_Element : constant Third_Party_Copyright_Type :=
           (Component_Name => new String'(Component_Name),
            Copyright_Notice => new String'(Copyright_Notice));
      begin
      Copyrights_Lists.Append_To
        (Vector => Third_Parties_Copyrights,
         Element => New_Element);
      end;

   end Add_Third_Party_Copyright_Notice;

   -- ========================================================================
   -- 2.3 Iterator on third parties copyright
   -- ------------------------------------------------------------------------

   procedure For_Each_Third_Party_Copyright
     (Accessor : in out Third_Party_Copyright_Accessor_Class)
   is
      Client_Accessor : Third_Party_Copyright_Accessor_Class renames Accessor;

      type Accessor_Wraper_Type is new Copyrights_Lists.Accessor_Type
      with null record;

      procedure Do_Access
        (Accessor : in out Accessor_Wraper_Type;
         Element  : in Third_Party_Copyright_Type)
      is
      begin
         pragma Unreferenced (Accessor);
         Do_Access
           (Accessor         => Client_Accessor,
            Component_Name   => Element.Component_Name.all,
            Copyright_Notice => Element.Copyright_Notice.all);
      end Do_Access;

      Accessor_Wraper : Accessor_Wraper_Type;
   begin
      Copyrights_Lists.For_Each_In
        (Accessor => Accessor_Wraper,
         Vector   => Third_Parties_Copyrights);
   end For_Each_Third_Party_Copyright;

   -- ========================================================================
   -- 3. Application Version
   -- ------------------------------------------------------------------------

   type Version_Type is record
      Major  : Natural;
      Minor  : Natural;
      Status : Version_Status_Kinds;
      Fixed  : Boolean := False;
   end record;

   -- ------------------------------------------------------------------------

   Version : Version_Type;

   -- ------------------------------------------------------------------------

   function Version_Fixed return Boolean
   is
   begin
      return Version.Fixed;
   end Version_Fixed;

   -- ------------------------------------------------------------------------

   procedure Set_Version
     (Major  : in Natural;
      Minor  : in Natural;
      Status : in Version_Status_Kinds)
   is
   begin
      Version.Major  := Major;
      Version.Minor  := Minor;
      Version.Status := Status;
      Version.Fixed  := True;
   end Set_Version;

   -- ------------------------------------------------------------------------

   function Version_Major return Natural
   is
   begin
      if not Version.Fixed then
         raise Program_Error;
      end if;

      return Version.Major;
   end Version_Major;

   -- ------------------------------------------------------------------------

   function Version_Minor return Natural
   is
   begin
      if not Version.Fixed then
         raise Program_Error;
      end if;

      return Version.Minor;
   end Version_Minor;

   -- ------------------------------------------------------------------------

   function Version_Status return Version_Status_Kinds
   is
   begin
      if not Version.Fixed then
         raise Program_Error;
      end if;

      return Version.Status;
   end Version_Status;

end Shell_UI.Application_Description;
