
-- Add to Shell UI, application description properties : name, purpose,
-- copyrights. These properties may be retrieved by an application or
-- may be used by Shell_UI.Help to display informations and help messages.

--! If you plan to use Shell_UI for personal use, don't forget to update
--! the Contact_E_Mail constant in this package;

package Shell_UI.Application_Description is

   -- ========================================================================
   -- Table of Content
   -- ------------------------------------------------------------------------
   -- 1. Application name and purpose
   -- 2. Application copyrights
   -- 3. Application version
   -- ------------------------------------------------------------------------

   Contact_E_Mail : constant String := "yannick_duchene@yahoo.fr";
   -- Change this whenever you use this package hierarchy for your own
   -- purpose. The contact e-mail adress is made a static constant, to
   -- make its inclusion automatic in all the applications released from a
   -- given author. Each software author using this package, thus have
   -- to properly change/setup this constant. This e-mail is displayed
   -- on apropos, infos and help standard command line options handling.
   -- Whenever you do so, you may simply add the original e-mail in a
   -- third-party copyright notice (see Shell_UI_Application_Pattern for
   -- an example of how to add a third party copyright notice).

   -- ========================================================================
   -- 1. Application name, purpose and copyrights
   -- ------------------------------------------------------------------------

   function Name_Fixed return Boolean;
   --|Initial: False

   function Purpose_Text_Fixed return Boolean;
   --|Initial: False

   function Name return String;
   --|Requires: Application_Name_Fixed

   function Purpose_Text return String;
   --|Ensures: not Application_Purpose_Text_Fixed -> Result = ""

   procedure Set_Name (Application_Name : in String);
   --|Requires: not Application_Name_Fixed
   --|Requires: Name'Length > 0
   --|Ensures: Application_Name_Fixed
   --|Ensures: Application_Name = Name

   procedure Set_Purpose_Text (Text : in String);
   --|Requires: not Application_Purpose_Text_Fixed
   --|Ensures: Application_Purpose_Text_Fixed
   --|Ensures: Application_Purpose_Text = Text

   -- ========================================================================
   -- 2. Application copyrights
   -- ------------------------------------------------------------------------

   function Main_Copyright_Notice_Fixed return Boolean;
   --|Initial: False

   function There_Are_Third_Parties_Copyrights return Boolean;
   --|Initial: False

   function Main_Copyright_Notice return String;
   --|Ensures: not Main_Copyright_Notice_Fixed -> Result = ""

   procedure Set_Main_Copyright_Notice (Copyright_Notice : in String);
   --|Requires: not Main_Copyright_Notice_Fixed
   --|Ensures: Main_Copyright_Notice_Fixed
   --|Ensures: Main_Copyright_Notice = Text

   procedure Add_Third_Party_Copyright_Notice
     (Component_Name   : in String;
      Copyright_Notice : in String);
   --|Requires: Main_Copyright_Notice_Fixed
   --|Ensures: There_Are_Third_Parties_Copyrights

   -- ------------------------------------------------------------------------

   type Third_Party_Copyright_Accessor_Type is abstract tagged private;

   procedure Do_Access
     (Accessor : in out Third_Party_Copyright_Accessor_Type;
      Component_Name   : in String;
      Copyright_Notice : in String)
   is abstract;

   subtype Third_Party_Copyright_Accessor_Class
     is Third_Party_Copyright_Accessor_Type'Class;

   procedure For_Each_Third_Party_Copyright
     (Accessor : in out Third_Party_Copyright_Accessor_Class);

   -- ========================================================================
   -- 2. Application Version
   -- ------------------------------------------------------------------------

   type Version_Status_Kinds is
     (Alpha,
      Beta,
      Pre_Release,
      Release);

   function Version_Fixed return Boolean;
   -- Initialy False

   procedure Set_Version
     (Major  : in Natural;
      Minor  : in Natural;
      Status : in Version_Status_Kinds);
   --|Ensures: Application_Version_Fixed

   function Version_Major return Natural;
   --|Requires: Application_Version_Fixed

   function Version_Minor return Natural;
   --|Requires: Application_Version_Fixed

   function Version_Status return Version_Status_Kinds;
   --|Requires: Application_Version_Fixed

   -- ========================================================================
   -- ***** DO NOT CROSS ***** DO NOT CROSS ***** DO NOT CROSS ***** DO NOT CR
   -- ========================================================================

private

   type Third_Party_Copyright_Accessor_Type is abstract tagged
     null record;

end Shell_UI.Application_Description;
