
-- Displays help and informations messages typical of a shell (command line)
-- application. Displays command line options helps, version, name, purpose
-- and the like.

-- Logically depends on :
-- with Command_Line_Options
-- with Application_Description

with Tiny.Texts;

package Shell_UI.Help is

   -- ========================================================================
   -- 1. Display help/usage and informations
   -- ------------------------------------------------------------------------

   procedure Display_Copyrights;
   --|Requires: Application_Description.Name_Fixed
   --|Requires: Application_Description.Main_Copright_Notice_Fixed
   -- Display of the main copyright, including third parties copyrights
   -- if there are some.
   -- Starts on a new line and ends with a new empty line

   procedure Display_Application_Version;
   --|Requires: Application_Description.Application_Name_Fixed
   --|Requires: Application_Description.Application_Version_Fixed
   -- Does not include copyrights informations.
   -- Starts on the current position and ends on a new empty line

   procedure Display_A_Propos;
   --|Requires: Application_Description.Application_Name_Fixed
   --|Requires: Application_Description.Application_Version_Fixed
   -- Includes copyrigths informations and application versions.
   -- Starts on a new line and ends with a new empty line

   procedure Display_Informations;
   --|Requires: Display_A_Propos.Requires
   -- If you are to display both usage and information, consider
   -- Display_Usage_And_Informations, which should be invoked instead
   -- of both separated methods.
   -- Includes applications version, purpose and copyrights.
   -- Starts on a new line and ends with a new empty line

   procedure Display_Usage (Message : in String := Tiny.Texts.Empty_String);
   --|Requires: Command_Line_Options.Command_Line_Parsed
   -- About the requirement Command_Line_Parsed : this is the only way to
   -- ensure all command line options to be defined was defined. As soon
   -- as the command line is parsed, declaration of command line options is
   -- closed. Displaying usage information, requires, to be menaingful, all
   -- command line options to be defined was defined. That's the reason
   -- of this requirement clause.
   -- If you are to display both usage and information, consider
   -- Display_Usage_And_Informations, which should be invoked instead
   -- of both separated methods.
   -- Starts on a new line and ends with a new empty line

   procedure Display_Usage_And_Informations;
   --|Requires:
   --|   Display_Application_Version.Requires and
   --|   Display_Usage.Requires
   -- Starts on a new line and ends with a new empty line

end Shell_UI.Help;
