
-- Add to Command_Line_Options, a quick to add typically used command line
-- options like "--help" and the like. This avoid to redefine these each
-- time and ensure consistency among applications created based on the
-- Shell_UI library.
--
-- Usage : use Command_Line_Options to define application specific command
-- line options, then use this to add standard options. Make to request
-- for Command_Line_Options to parse the command line, then make a request
-- to this package to handle standard options *before* you handle your
-- application standard options. If a standard option direct the application
-- to terminates right after it has handled the standard option, then the
-- package will raise Quit_Application. Client of this package should
-- handle this exceptio so that when it occurs, the application do nothing
-- and silently terminates.

package Shell_UI.Standard_Command_Line_Options is

   procedure Add_Standard_Options;
   -- This is named Add_Standard_Option to express a usage recommendation :
   -- The Command_Line_Options.Declare_Option function, expect the name
   -- of an option which is not already declared (either its short name or
   -- its long name). Using standard option, may conflict with application
   -- specific option (although it is not recommanded to re-use standard
   -- option names for other purpose, these are not reserved). So, standard
   -- options required to be registered after the application has register
   -- its own options. There is not requirement for this, as this cannot
   -- be checked, let say this is strongly recommanded. That's why this
   -- method is named Add_Standard_Options, because standard options are to
   -- be added to the one the application defines.
   -- This method will check, for each standard options, if the application
   -- has already registered an option of the same short/long name. If it
   -- is, the method will not register the corresponding standard option, and
   -- the later Handle_Standard_Options will not make interpretation of the
   -- corresponding option.
   --|Requires: not Standard_Command_Line_Options.Command_Line_Parsed
   -- This method currently declare the following command line options :
   -- 1) -h --help
   -- 2) -v --version
   -- 3)    --infos
   -- 4)    --apropos
   -- 5)    --copyrights

   Quit_Application : exception;
   -- This is exception is raised to quit the application, whenever
   -- the handling of a standard option direct the application to quit
   -- immediately. The main program should set an handler to catch this
   -- exception and this handler should either do nothing or return.

   procedure First_Handle_Standard_Options;
   -- Just like the way Add_Standard_Options is named, the name of this
   -- method express another usage recommandation : it should be invoked
   -- before the application makes interpretation of its own options. This
   -- is because the interpretation of a standard option may requires the
   -- application to terminate immediately, as an exemple, when the user
   -- typed --help and Add_Standard_Options was invoked at application
   -- options initilization, then First_Handle_Standard_Options will
   -- output the Help-and-Usage message and quit the application.
   --|Requires: Standard_Command_Line_Options.Command_Line_Parsed
   -- The application will terminates on invokation of
   -- First_Handle_Standard_Options whenever the following options
   -- was registered by Add_Standard_Options and found in the command line :
   -- 1) -h --help
   -- 3)    --infos

end Shell_UI.Standard_Command_Line_Options;
