
-- This is a pattern for application based on the Shell_UI library.
-- Do not modify this pattern : make a copy and give it a meaningful name
-- instead. Locations to be edited or reviewed are marked with comments
-- starting with "--!"

--! If you plan to use Shell_UI for personal use, don't forget to update
--! the Contact_E_Mail constant in Shell_UI.Application_Description.

with Shell_UI.Application_Description;
with Shell_UI.Command_Line_Options;
with Shell_UI.Standard_Command_Line_Options;
with Shell_UI.Help;

with Ada.Text_IO;

procedure Shell_UI_Application_Pattern
is
   package IO renames Ada.Text_IO;
begin

   Application_Description :
   declare
      package Application_Description renames
        Shell_UI.Application_Description;

      Application_Name : constant String :=
        "My Application Name";
      --! To be edited

      Application_Purpose : constant String :=
        "My Application Purpose";
      --! To be edited

      Main_Copyrigth : constant String :=
        "My Application Copyrights";
      --! To be edited

      Third_Party_Component_Name : constant String :=
        "A Third Party Component Name";
      --! To be edited

      Third_Party_Copyright : constant String :=
        "The Third Party Component Copyright";
      --! To be edited

   begin
      Application_Description.Set_Name
        (Application_Name);

      Application_Description.Set_Purpose_Text
        (Application_Purpose);

      Application_Description.Set_Main_Copyright_Notice
        (Main_Copyrigth);

      Application_Description.Add_Third_Party_Copyright_Notice
        (Component_Name   => Third_Party_Component_Name,
         Copyright_Notice => Third_Party_Copyright);
      --! To be edited

      Application_Description.Set_Version
        (Major  => 1,
         Minor  => 0,
         Status => Application_Description.Release);
      --! To be edited

   end Application_Description;

   Command_Line_Interpretation :
   declare

      package Options renames Shell_UI.Command_Line_Options;
      package Standard_Options renames Shell_UI.Standard_Command_Line_Options;
      package Help renames Shell_UI.Help;

      Option_1_Short_Name : constant Character := 'o';
      Option_1_Long_Name  : constant String := "my-option";
      Is_Option_1_A_Flag  : constant Boolean := False;
      Option_1_Type_Name  : constant String := "type";
      Option_1_Help       : constant String :=
        ("My option purpose and usage.");
      --! To be edited

      --! Add other options as needed

      Message_When_Something_Is_Not_Ok : constant String :=
          ("My message when something is not ok.");
      --! To be edited

      --! Add other message strings as needed

      Option_1 : constant Options.Option_Access :=
        Options.Declare_Option
          (Short_Name => Option_1_Short_Name,
           Long_Name  => Option_1_Long_Name,
           Is_A_Flag  => Is_Option_1_A_Flag,
           Type_Name  => Option_1_Type_Name,
           Help_Text  => Option_1_Help);
      --! To be edited

      --! Add other options as needed

   begin
      Standard_Options.Add_Standard_Options;
      Options.Parse_Command_Line;
      Standard_Options.First_Handle_Standard_Options;

      if Options.Provided (Option_1) then
         -- Nice user invoked me, we can do the job together
         IO.New_Line;
         IO.Put ("You're a good boy/girl, you gave me """);
         IO.Put (Options.Value (Option_1));
         IO.Put ("""");
         IO.New_Line;
      -- elsif
         --! Add other handlings and interpretations as needed
      else
         Help.Display_Usage
           (Message => Message_When_Something_Is_Not_Ok);
      end if;

   end Command_Line_Interpretation;

exception
   when Shell_UI.Standard_Command_Line_Options.Quit_Application =>
      null;
      -- A normal request to qui the application : no notification.
   when Shell_UI.Command_Line_Options.Command_Line_Error =>
      null;
      -- The Command_Line_Options package has already displayed a
      -- message : do not add more.
end Shell_UI_Application_Pattern;
