
package body Tiny.Containers is

   package body Vectors is

      -- =====================================================================
      -- Table of Content
      -- ---------------------------------------------------------------------
      -- 1. Vector type
      -- 2. Tester object : no implementation
      -- 3. Testing iteration
      -- 4. Requester object : no implementation
      -- 5. Request iteration
      -- 6. Updater object : no implementation
      -- 7. Update iteration
      -- ---------------------------------------------------------------------

      -- =====================================================================
      -- 1. Vector type
      -- ---------------------------------------------------------------------
      -- 1.1 Node type
      -- 1.2 Vector type
      -- 1.3 Vector methods
      -- ---------------------------------------------------------------------

      -- ---------------------------------------------------------------------
      -- 1.1 Node type
      -- ---------------------------------------------------------------------

      type Node_Type;

      ------------------------------------------------------------------------

      type Node_Access is access Node_Type;

      ------------------------------------------------------------------------

      type Node_Type is record
         Value     : Element_Type;
         Next_Node : Node_Access;
      end record;

      -- ---------------------------------------------------------------------
      -- 1.2 Vector type
      -- ---------------------------------------------------------------------

      type Vector_Type is record
         First : Node_Access := null;
         Last  : Node_Access := null;
      end record;

      -- ---------------------------------------------------------------------
      -- 1.3 Vector methods
      -- ---------------------------------------------------------------------

      function Is_Null
        (Vector : Vector_Access)
         return Boolean
      is
      begin
         return Vector = Null_Vector;
      end Is_Null;

      ------------------------------------------------------------------------

      procedure Append_To
        (Vector : in out Vector_Access;
         Node   : in Node_Access)
      is
      begin
         if Vector = Null_Vector then
            Vector := new Vector_Type;
            Vector.First := Node;
            Vector.Last := Node;
            Node.Next_Node := null;
         else
            Vector.Last.Next_Node := Node;
            Vector.Last := Node;
            Node.Next_Node := null;
         end if;
      end Append_To;

      ------------------------------------------------------------------------

      procedure Append_To
        (Vector  : in out Vector_Access;
         Element : in Element_Type)
      is
         Node : Node_Access := new Node_Type;
      begin
         Node.Value := Element;
         Append_To (Vector => Vector, Node => Node);
      end Append_To;

      -- =====================================================================
      -- 3. Testing iteration
      -- ---------------------------------------------------------------------

      function Exist_In
        (Vector : Vector_Access;
         Test   : Test_Class)
         return Boolean is
      begin

         Empty_List :
         begin
            if Vector = Null_Vector then
               return False;
            end if;
         end Empty_List;

         None_Empty_List :
         declare
            Iterated_Node : Node_Access := Vector.First;
         begin
            Iterator :
            while Iterated_Node /= null loop
               if Checked (Test => Test, Element => Iterated_Node.Value) then
                  return True;
               end if;
               Iterated_Node := Iterated_Node.Next_Node;
            end loop Iterator;
            return False;
         end None_Empty_List;

      end Exist_In;

      -- ---------------------------------------------------------------------

      procedure Find_In
        (Vector  : in Vector_Access;
         Test    : in Test_Class;
         Found   : out Boolean;
         Element : out Element_Type)
      is
      begin

         Empty_List :
         begin
            if Vector = Null_Vector then
               Found := False;
               return;
            end if;
         end Empty_List;

         None_Empty_List :
         declare
            Iterated_Node : Node_Access := Vector.First;
         begin
            Iterator :
            while Iterated_Node /= null loop
               if Checked (Test => Test, Element => Iterated_Node.Value) then
                  Found := True;
                  Element := Iterated_Node.Value;
                  return;
               end if;
               Iterated_Node := Iterated_Node.Next_Node;
            end loop Iterator;
            Found := False;
         end None_Empty_List;

      end Find_In;

      -- =====================================================================
      -- 5. Request iteration
      -- ---------------------------------------------------------------------

      procedure For_Each_In
        (Vector   : in Vector_Access;
         Accessor : in out Accessor_Class)
      is
      begin

         Empty_List :
         begin
            if Vector = Null_Vector then
               return;
            end if;
         end Empty_List;

         None_Empty_List :
         declare
            Iterated_Node : Node_Access := Vector.First;
         begin
            Iterator :
            while Iterated_Node /= null loop
               Do_Access
                 (Accessor => Accessor,
                  Element  => Iterated_Node.Value);
               Iterated_Node := Iterated_Node.Next_Node;
            end loop Iterator;
         end None_Empty_List;

      end For_Each_In;

      -- =====================================================================
      -- 7. Update iteration
      -- ---------------------------------------------------------------------

      procedure For_Each_In
        (Vector   : in Vector_Access;
         Modifier : in out Modifier_Class)
      is
      begin

         Empty_List :
         begin
            if Vector = Null_Vector then
               return;
            end if;
         end Empty_List;

         None_Empty_List :
         declare
            Iterated_Node : Node_Access := Vector.First;
         begin
            Iterator :
            while Iterated_Node /= null loop
               Modify
                 (Modifier => Modifier,
                  Element  => Iterated_Node.Value);
               Iterated_Node := Iterated_Node.Next_Node;
            end loop Iterator;
         end None_Empty_List;

      end For_Each_In;

   end Vectors;

end Tiny.Containers;
